﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace GFramework.Base
{
    public interface IPositioned
    {
        Vector3 Position { get; set; }
        event EventHandler<PositionChangedEventArgs> PositionChanged;
    }

    public interface IDirected:IPositioned
    {
        Vector3 Direction { get; set; }

        void Rotate(Vector3 axisRotation);
        void Rotate(float xrotation, float yrotation, float zrotation);

        void StrafeLeft(float distance);
        void StrafeRight(float distance);
        void MoveForward(float distance);
        void MoveBackward(float distance);

        event EventHandler<PositionChangedEventArgs> DirectionChanged;
    }

    public class PositionChangedEventArgs : EventArgs
    {
        public PositionChangedEventArgs(Vector3 oldPos, Vector3 newPos)
        {
            OldValue = oldPos;
            NewValue = newPos;
        }

        public Vector3 OldValue { get; private set; }
        public Vector3 NewValue { get; private set; }
    }

    public interface IGameManager
    {
         Game Game { get;}
    }

    public interface ITerrain
    {
        float GetHeight(float x, float y);
    }

    public interface ICamera: IPositioned
    {
        Guid ID { get; }

        Matrix View { get; }
        Matrix Projection { get; }
        BoundingFrustum Frustum { get; }
    }

    public interface ICameraManager
    {
        
    }
}
