﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GFramework.Base;
using Microsoft.Xna.Framework;

namespace GFramework.Camera
{
    public class CameraManager
    {
        private Dictionary<Guid,ICamera> _cameras;
        private Guid _currentCamId;

        public CameraManager()
        {
            _cameras = new Dictionary<Guid, ICamera>();
        }

        public ICamera AddDefaultCamera(Vector3 position, Vector3 target)
        {
            Camera cam = new Camera(position, target);
            _cameras[cam.ID] = cam;
            return cam;
        }

        public ICamera AddFollowingCamera(IDirected origin)
        {
            FollowingCamera cam = new FollowingCamera(origin);
            _cameras[cam.ID] = cam;
            return cam;
        }

        public ICamera AddMouseCamera(Vector3 position,Vector3 target)
        {
            MouseCamera cam = new MouseCamera(position, target);
            _cameras[cam.ID] = cam;
            return cam;
        }

        public ICamera Current
        {
            get
            {
                if (_currentCamId==Guid.Empty)
                {
                    if (_cameras.Count > 0)
                        _currentCamId = _cameras.First().Key;
                }
                if (_cameras.ContainsKey(_currentCamId))
                    return _cameras[_currentCamId];
                throw new Exception("No valid camera set as Current in CameraManager");
            }
        }
    }
}
