﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GFramework.Base;
using Microsoft.Xna.Framework;

namespace GFramework.Camera
{
    /// <summary>
    /// Проксирующий по сути камеру объект, связывающий камеру и другой объект с интерфейсом  IDirected
    /// </summary>
    public class FollowingCamera:ICamera
    {
        private Camera _camera;
        private IDirected _source;

        #region Конструкторы
        public FollowingCamera(IDirected source)
        {
            _source = source;
            _camera = new Camera(source.Position, source.Position+source.Direction);
            
        }

        public FollowingCamera(IDirected source, float maxDistance, float minDistance)
        {
            _source = source;
            _camera = new Camera(source.Position, source.Position + source.Direction, maxDistance, minDistance);
            SetupEvents();
        }
        #endregion

        private void SetupEvents()
        {
            _source.PositionChanged += source_PositionChanged;
            _source.DirectionChanged += source_DirectionChanged;
        }

        private void source_DirectionChanged(object sender, PositionChangedEventArgs e)
        {
            _camera.Direction = e.NewValue;
        }

        private void source_PositionChanged(object sender, PositionChangedEventArgs e)
        {
            _camera.Position = e.NewValue;
        }


        public Guid ID
        {
            get { return _camera.ID; }
        }

        public Matrix View
        {
            get { return _camera.View; }
        }

        public Matrix Projection
        {
            get { return _camera.Projection; }
        }

        public BoundingFrustum Frustum
        {
            get { return _camera.Frustum; }
        }

    	public Vector3 Position
    	{
			get { return _source.Position; }
			set { _source.Position = value; }
    	}

    	public event EventHandler<PositionChangedEventArgs> PositionChanged;
    }
}
