﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GFramework.Base;
using GFramework.Input;
using GFramework.UI;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Keyboard=GFramework.Input.Keyboard;

namespace GFramework.Camera
{
    public class MouseCamera:Camera
    {
        public MouseCamera(Vector3 position, Vector3 target, float minDistance, float maxDistance) : base(position, target, minDistance, maxDistance)
        {
            SetupEvents();
        }

        private void SetupEvents()
        {
            GameManager.Instance.MouseManager.MouseMoved += new EventHandler<GFramework.Input.MouseMovedEventArgs>(MouseManager_MouseMoved);
            GameManager.Instance.MouseManager.MouseButtonsPressed += new EventHandler<GFramework.Input.MouseButtonEventArgs>(MouseManager_MouseButtonsPressed);
            GameManager.Instance.MouseManager.MouseButtonsReleased += new EventHandler<GFramework.Input.MouseButtonEventArgs>(MouseManager_MouseButtonsReleased);
            GameManager.Instance.MouseManager.MouseWheelScrolled += new EventHandler<MouseScrolledEventArgs>(MouseManager_MouseWheelScrolled);

            //UserInterface.UI.Keyboard.KeysDown += new EventHandler<Keyboard.KeyboarKeyEventArgs>(Keyboard_KeysDown);
            GameManager.Instance.Keyboard.KeyPressed += new EventHandler<KeyboardCharEventArgs>(Keyboard_KeyPressed);
        }

        private const float MOVE_STEP = 25;
        private void Keyboard_KeyPressed(object sender, KeyboardCharEventArgs e)
        {
            if (e.Keys.Contains(Keys.Right))
                StrafeRight(MOVE_STEP);
            if (e.Keys.Contains(Keys.Left))
                StrafeRight(-MOVE_STEP);
            if (e.Keys.Contains(Keys.Up))
                MoveForward(MOVE_STEP);
            if (e.Keys.Contains(Keys.Down))
                MoveBackward(MOVE_STEP);
        }

        void Keyboard_KeysDown(object sender, KeyboarKeyEventArgs e)
        {
            if (e.Keys.Contains(Keys.Right))
                StrafeRight(0.25f);
            if (e.Keys.Contains(Keys.Left))
                StrafeRight(-0.25f);
            if (e.Keys.Contains(Keys.Up))
                MoveForward(0.25f);
            if (e.Keys.Contains(Keys.Down))
                MoveBackward(0.25f);
        }

        void MouseManager_MouseWheelScrolled(object sender, MouseScrolledEventArgs e)
        {
            MoveForward(e.Delta/50);
        }

        void MouseManager_MouseButtonsReleased(object sender, GFramework.Input.MouseButtonEventArgs e)
        {
            if ((e.Buttons & MouseButtons.Right) == MouseButtons.Right)
                moving = false;
        }

        private bool moving;
        void MouseManager_MouseButtonsPressed(object sender, GFramework.Input.MouseButtonEventArgs e)
        {
           if ( (e.Buttons & MouseButtons.Right) == MouseButtons.Right)
               moving = true;
        }

        void MouseManager_MouseMoved(object sender, GFramework.Input.MouseMovedEventArgs e)
        {
            if (moving)
                this.Rotate(e.MovedDelta.Y/360,e.MovedDelta.X/360,0);
        }



        public MouseCamera(Vector3 position, Vector3 target) : base(position, target)
        {
            SetupEvents();
        }


        
    }
}
