﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GFramework.Base;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GFramework.Environment
{
    public class Environment:DrawableGameComponent
    {
        private RenderTarget2D _environmentRenderT;

        private Terrain _terrain;
        private Texture2D _terrainTexture;

        public Environment(Game game):base(game)
        {
            DrawOrder = 1;
        }

        protected override void LoadContent()
        {
            _terrainTexture = Game.Content.Load<Texture2D>("heightmap");
            _terrain = new Terrain(_terrainTexture);
        }

        public override void Update(GameTime gameTime)
        {
            _terrain.Update(gameTime);
        }

        private void ResetRenderStates()
        {
            GraphicsDevice.RenderState.DepthBufferEnable = true;
            GraphicsDevice.RenderState.AlphaBlendEnable = false;
            GraphicsDevice.RenderState.AlphaTestEnable = false;
            GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
            //GraphicsDevice.RenderState.CullMode = CullMode.None;
            GraphicsDevice.RenderState.AlphaSourceBlend = Blend.SourceAlpha;
            GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
        }

        public override void Draw(GameTime gameTime)
        {
            var oldState = GraphicsDevice.RenderState.FillMode;
            //GraphicsDevice.RenderState.DepthBufferEnable = true;
            ResetRenderStates();
         
            GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
            GraphicsDevice.Clear(Color.CornflowerBlue);
            
            _terrain.Draw(gameTime,GameManager.Instance.CameraManager.Current);
            GraphicsDevice.RenderState.FillMode = oldState;
        }
    }
}
