﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GFramework.Base;
using GFramework.UI;
using Microsoft.Xna.Framework.Input;

namespace GFramework.Input
{
    public class ActionKeyMapper
    {
        private Dictionary<string, Keys[]> _keysDict;
        private Dictionary<string, int> _actionState;

        public event EventHandler<ActionEventArgs> ActionFired;

        public ActionKeyMapper()
        {
            _keysDict = new Dictionary<string, Keys[]>();
            _actionState = new Dictionary<string, int>();
        }

        public void Initialize()
        {
            GameManager.Instance.Keyboard.KeysDown += InputEventSender_keyboardKeyPressed;
            GameManager.Instance.Keyboard.KeysUp += InputEventSender_keyboardKeyReleased;
        }

        public bool AddAction(string name, params Keys[] keys)
        {
            if (keys.Length != (keys.Distinct()).Count())
                return false;
            _keysDict[name] = keys;
            return true;
        }

        private void InputEventSender_keyboardKeyReleased(object o, KeyboarKeyEventArgs e)
        {
            if (!Enabled)
                return;
            Keys key = e.Keys.First();
            foreach (KeyValuePair<string, Keys[]> dict in _keysDict)
            {
                if (_actionState.ContainsKey(dict.Key))
                {
                    for (int i = 0; i < _actionState[dict.Key]; i++)
                    {
                        if (dict.Value[i] == key)
                        {
                            _actionState.Remove(dict.Key);
                            break;
                        }
                    }
                }
            }
        }

        void InputEventSender_keyboardKeyPressed(object o, KeyboarKeyEventArgs e)
        {
            if (!Enabled)
                return;
            Keys key = e.Keys.First();
            foreach (KeyValuePair<string, Keys[]> dict in _keysDict)
            {
                if (_actionState.ContainsKey(dict.Key))
                {
                    int state = _actionState[dict.Key];
                    int maxCount = dict.Value.Length - 1;
                    if (state == maxCount && dict.Value[maxCount] == key)
                    {
                        FireAction(dict.Key);
                    }
                    else
                        if (state < maxCount && dict.Value[state + 1] == key)
                            _actionState[dict.Key]++;
                }
                else
                {
                    if (dict.Value[0] == key)
                        _actionState[dict.Key] = 0;
                }
            }
        }

        public bool RemoveAction(string name)
        {
            if (_keysDict.ContainsKey(name))
            {
                _keysDict.Remove(name);
                return true;
            }
            return false;
        }

        private void FireAction(string name)
        {
            if (ActionFired != null)
            {
                ActionEventArgs arg = new ActionEventArgs(name);
                ActionFired(this, arg);
            }
        }

        public void AddDefaultActions()
        {
            AddAction(ActionNames.FORWARD, Keys.W);
            AddAction(ActionNames.BACKWARD, Keys.S);
            AddAction(ActionNames.STRAFERIGHT, Keys.D);
            AddAction(ActionNames.STRAFELEFT, Keys.A);

            AddAction(ActionNames.EXIT1, Keys.LeftAlt, Keys.X);
            AddAction(ActionNames.EXIT2, Keys.LeftAlt, Keys.Q);
            AddAction(ActionNames.EXIT3, Keys.Escape);
        }

        bool _enabled = true;
        public bool Enabled
        {
            get
            {
                return _enabled;
            }
            set
            {
                _enabled = value;
                if (!_enabled)
                    _actionState.Clear();
            }
        }
    }

    public class ActionEventArgs : EventArgs
    {
        public ActionEventArgs(string name)
        {
            ActionName = name;
        }

        public string ActionName { get; private set; }
    }
}
