﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using GFramework.Input.Languages;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace GFramework.Input
{
    public class Keyboard
    {
        private KeyboardState _lastState, _currState;
        private bool _shiftDown;
        private float _holdTimer, _holdWait = 100;

        public event EventHandler<KeyboarKeyEventArgs> KeysDown;
        public event EventHandler<KeyboarKeyEventArgs> KeysUp;

        public event EventHandler<KeyboardCharEventArgs> KeyPressed;

        public void Update(GameTime time)
        {
            _currState = Microsoft.Xna.Framework.Input.Keyboard.GetState();

            _shiftDown = _currState.IsKeyDown(Keys.LeftShift) || _currState.IsKeyDown(Keys.RightShift);

            var wasPressed = _lastState.GetPressedKeys();
            var nowPressed = _currState.GetPressedKeys();
            
            var justPressed = nowPressed.Except(wasPressed);
            var justReleased = wasPressed.Except(nowPressed);
            var stillPressed = nowPressed.Except(justPressed);

            if (KeysDown!=null && justPressed.Count()>0)
            {
                KeyboarKeyEventArgs args = new KeyboarKeyEventArgs(justPressed.ToArray());
                KeysDown(this,args);
            }

            if (KeysUp != null && justReleased.Count()>0)
            {
                KeyboarKeyEventArgs args = new KeyboarKeyEventArgs(justReleased.ToArray());
                KeysUp(this, args);
            }

            _holdTimer += time.ElapsedRealTime.Milliseconds;
            if (KeyPressed!=null && stillPressed.Count()>0 && _holdTimer>_holdWait)
            {
                char? ch = KeyToChar(stillPressed.Last());
                KeyboardCharEventArgs args = new KeyboardCharEventArgs(ch, stillPressed.ToArray());
                KeyPressed(this, args);
                _holdTimer = 0;
            }

            _lastState = _currState;
        }

        public char? KeyToChar(Keys key)
        {
            byte keyByte = (byte)key;

            if (Thread.CurrentThread.CurrentUICulture.TwoLetterISOLanguageName.ToLower() != "ru" && ( keyByte > 64 && keyByte < 91))   // a letter
                return _shiftDown ? (char)key : char.ToLower((char)key);

            if (RuKeyHelper.Mapping.ContainsKey(key))
                return RuKeyHelper.Mapping[key];

            if (keyByte > 47 && keyByte < 58)
            {
                switch ((char)key)
                {
                    case '0': return _shiftDown ? ')' : '0';
                    case '1': return _shiftDown ? '!' : '1';
                    case '2': return _shiftDown ? '@' : '2';
                    case '3': return _shiftDown ? '#' : '3';
                    case '4': return _shiftDown ? '$' : '4';
                    case '5': return _shiftDown ? '%' : '5';
                    case '6': return _shiftDown ? '^' : '6';
                    case '7': return _shiftDown ? '&' : '7';
                    case '8': return _shiftDown ? '*' : '8';
                    case '9': return _shiftDown ? '(' : '9';
                }
            }

            switch (key)
            {
                case Keys.Space: return ' ';
                case Keys.Decimal: return '.';
                case Keys.Add: return '+';
                case Keys.Subtract: return '-';
                case Keys.OemSemicolon: return _shiftDown ? ':' : ';';
                case Keys.OemQuotes: return _shiftDown ? '"' : '\'';
                case Keys.OemPeriod: return _shiftDown ? '>' : '.';
                case Keys.OemComma: return _shiftDown ? '<' : ',';
                case Keys.OemQuestion: return _shiftDown ? '?' : '/';
                case Keys.OemPipe: return _shiftDown ? '|' : '\\';
                case Keys.OemPlus: return _shiftDown ? '+' : '=';
                case Keys.OemMinus: return _shiftDown ? '_' : '-';
                case Keys.OemOpenBrackets: return _shiftDown ? '{' : '[';
                case Keys.OemCloseBrackets: return _shiftDown ? '}' : ']';
                case Keys.OemTilde: return _shiftDown ? '~' : '`';
                default: return null;
            };
        }
    }

    public class KeyboarKeyEventArgs : EventArgs
    {
        public Keys[] Keys { get; private set; }

        public KeyboarKeyEventArgs(Keys[] keys)
        {
            Keys = keys;
        }
    }

    public class KeyboardCharEventArgs : EventArgs
    {
        public char? Char { get; private set; }
        public Keys[] Keys { get; private set; }

        public KeyboardCharEventArgs(char? key, Keys[] kSt)
        {
            Char = key;
            Keys = kSt;
        }
    }
}
