﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using Microsoft.Xna.Framework.Graphics;

namespace GFramework.Vertices
{
    public class ListConnectedIndexBuffer<T> where T:struct 
    {
        private IList<T> _source;
        private DynamicIndexBuffer _buffer;
        private GraphicsDevice _device;
        private int _elementSize;

        public ListConnectedIndexBuffer(IList<T> indexList, GraphicsDevice device)
        {
            _source = indexList;
            _device = device;
            _buffer = new DynamicIndexBuffer(device,typeof(T),_source.Count,BufferUsage.WriteOnly);
            _buffer.SetData(_source.ToArray(), 0, _source.Count, SetDataOptions.Discard);
            _buffer.ContentLost += new EventHandler(Buffer_ContentLost);
            _elementSize = Marshal.SizeOf(this.GetType().GetGenericArguments()[0]);
        }

        void Buffer_ContentLost(object sender, EventArgs e)
        {
            ResetBuffer();
            _buffer.SetData(_source.ToArray(),0,_source.Count,SetDataOptions.Discard);
        }

        private void ResetBuffer()
        {
            _buffer.Dispose();
            _buffer = new DynamicIndexBuffer(_device, typeof(T), _source.Count, BufferUsage.WriteOnly);
        }

        public void UpdateBuffer()
        {
            if ( _buffer.SizeInBytes / _elementSize < _source.Count)
            {
                ResetBuffer();
            }
            _buffer.SetData(_source.ToArray(), 0, _source.Count, SetDataOptions.Discard);
        }

        public DynamicIndexBuffer Buffer
        {
            get { return _buffer;}
        }
    }
}
